@@ -1,13 +1,15 @@
 ;;; mu4e-alert.el --- Desktop notification for mu4e  -*- lexical-binding: t; -*-
 
 ;; Copyright (C) 2015-2017  Iqbal Ansari
+;; Copyright (C) 2021-2024  Mikhail Rudenko
 
 ;; Author: Iqbal Ansari <iqbalansari02@yahoo.com>
-;; URL: https://github.com/iqbalansari/mu4e-alert
+;; Maintainer: Mikhail Rudenko <mike.rudenko@gmail.com>
+;; URL: https://github.com/xzz53/mu4e-alert
 ;; Keywords: mail, convenience
-;; Package-Version: 1.0
-;; Package-Revision: 3453e25ff6c0
-;; Package-Requires: ((alert "1.2") (s "1.10.0") (ht "2.0") (emacs "24.1"))
+;; Package-Version: 20251022.2130
+;; Package-Revision: 9f20f30b15a5
+;; Package-Requires: ((alert "1.2") (s "1.10.0") (ht "2.0") (emacs "24.4"))
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -22,14 +24,14 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
+
 
 ;;; Commentary:
 
 ;; This package provides desktop notifications for mu4e, additionally it can
 ;; display the number of unread emails in the modeline
 
-
+
 
 ;;; Code:
 
@@ -44,12 +46,10 @@
 (require 'pcase)
 (require 'cl-lib)
 
-
-
 ;; Customizations
 
 (defgroup mu4e-alert nil
-  "Customization options for mu4e-alert"
+  "Customization options for mu4e-alert."
   :group 'mail
   :prefix "mu4e-alert-")
 
@@ -65,7 +65,7 @@ be string to be sent to the mu's find command."
   #'mu4e-alert-default-mode-line-formatter
   "The function used to get the string to be displayed in the mode-line.
 It should be a function that accepts a single argument the current count of
-unread emails and should return the string to be displayed in the mode-line"
+unread emails and should return the string to be displayed in the mode-line."
   :type 'function
   :group 'mu4e-alert)
 
@@ -73,7 +73,7 @@ unread emails and should return the string to be displayed in the mode-line"
   #'mu4e-alert-default-email-count-notification-formatter
   "The function used to get the message for the desktop notification.
 It should be a function that accepts a single argument the current count of
-unread emails and should return the string to be used for the notification"
+unread emails and should return the string to be used for the notification."
   :type 'function
   :group 'mu4e-alert)
 
@@ -107,11 +107,13 @@ messages should grouped together in one notification."
   "The function used to get arrange similar mails in to a group.
 
 It should accept a list of mails and return a list of lists, where each list is
-a group of messages that user should be notified about in one notification.")
+a group of messages that user should be notified about in one notification."
+  :type 'function)
 
 (defcustom mu4e-alert-grouped-mail-sorter
   #'mu4e-alert-default-grouped-mail-sorter
-  "The function used to sort the emails after grouping them.")
+  "The function used to sort the emails after grouping them."
+  :type 'function)
 
 (defcustom mu4e-alert-grouped-mail-notification-formatter
   #'mu4e-alert-default-grouped-mail-notification-formatter
@@ -119,7 +121,8 @@ a group of messages that user should be notified about in one notification.")
 
 The function is used get the notification to be displayed for a group of emails.
 It should return a plist with keys :title and :body with the value of title and
-body for the notification respectively.")
+body for the notification respectively."
+  :type 'function)
 
 (defcustom mu4e-alert-email-notification-types '(count subjects)
   "The types of notifications to be displayed for emails.
@@ -129,12 +132,14 @@ have following elements
 count    - Notify the total email count to the user
 subjects - Notify with some content of the email, by default the emails are
            grouped by the sender.  And one notification is issued per sender
-           with the subject of the emails is displayed in the notification.")
+           with the subject of the emails is displayed in the notification."
+  :type 'symbol)
 
 (defcustom mu4e-alert-set-window-urgency t
   "Set window urgency on recieving unread emails.
 
-If non-nil `mu4e-alert' will set the WM_URGENT on detecting unread messages")
+If non-nil `mu4e-alert' will set the WM_URGENT on detecting unread messages."
+  :type 'boolean)
 
 (defcustom mu4e-alert-notify-repeated-mails nil
   "Notify about interesting mails that were notified about earlier.
@@ -145,7 +150,15 @@ the other unread, next time the when mu4e-alert checks for unread emails, it
 will filter out the second message and show notifications only for mails that
 have arrived after the first check.  Set this option to a non-nil value if you
 wish to be notified of all emails at each check irrespective of whether you have
-been notified of the an email earlier or no.")
+been notified of the an email earlier or no."
+  :type 'boolean)
+
+(defcustom mu4e-alert-icon nil
+  "Full path to icon to show in notification.
+
+Note that not all 'alert' notification backends support this."
+  :type 'string
+  :group 'mu4e-alert)
 
 ;;;###autoload
 (defun mu4e-alert-set-default-style (value)
@@ -175,7 +188,48 @@ See also https://github.com/jwiegley/alert."
   :set (lambda (_ value) (mu4e-alert-set-default-style value))
   :group 'mu4e-alert)
 
-
+;; deal with mu4e versions above 1.7
+;; need to know whether on mu4e 1.7.x because its API changed.
+(defconst mu4e-alert--mu-is-17
+  (version-list-<= '(1 7) (version-to-list mu4e-mu-version))
+  "Non-nil if we are using mu 1.7.4 or newer.")
+
+;; declare functions to keep byte-compiler quiet
+(declare-function mu4e-running-p "mu4e-server" nil t)
+(declare-function mu4e~proc-running-p "mu4e" nil t)
+(declare-function mu4e--server-find "mu4e-server" nil t)
+(declare-function mu4e~proc-find "mu4e" nil t)
+(declare-function mu4e--init-handlers "mu4e" nil t)
+(declare-function mu4e-search "mu4e-search" nil t)
+
+;; supress byte-compiler warning in mu4e v1.7
+(eval-when-compile
+  (cl-remprop 'mu4e-headers-search 'byte-obsolete-info))
+
+;; make sure mu4e handlers are initialized in v1.7
+(when mu4e-alert--mu-is-17
+  (mu4e--init-handlers))
+
+;; create aliases for mu4e functions and variables
+(defvaralias 'mu4e-alert-mu4e-header-func-var
+  (if mu4e-alert--mu-is-17 'mu4e-headers-append-func 'mu4e-header-func)
+  "mu4e variable storing function to process emails in header view.")
+
+(defvaralias 'mu4e-alert--mu4e-context-current
+  (if mu4e-alert--mu-is-17 'mu4e--context-current 'mu4e~context-current)
+  "mu4e variables to store the current context.")
+
+(defalias 'mu4e-alert--mu4e-search
+  (if mu4e-alert--mu-is-17 #'mu4e-search #'mu4e-headers-search)
+  "mu4e function to search for mail.")
+
+(defalias 'mu4e-alert--mu4e-proc-running-p-func
+  (if mu4e-alert--mu-is-17 #'mu4e-running-p #'mu4e~proc-running-p)
+  "Function that checks whether mu4e process is running.")
+
+(defalias 'mu4e-alert--mu4e-proc-find-func
+  (if mu4e-alert--mu-is-17 #'mu4e--server-find #'mu4e~proc-find)
+  "Function that searches for mail using mu4e.")
 
 ;; Core functions
 
@@ -189,59 +243,62 @@ See also https://github.com/jwiegley/alert."
                (file-executable-p mu4e-mu-binary))
     (user-error "Please set `mu4e-mu-binary' to the full path to the mu binary, before attempting to enable `mu4e-alert'")))
 
-(defun mu4e-alert--parse-mails (buffer)
-  "Parse the emails in BUFFER.
-The buffer holds the emails received from mu in sexp format"
-  (read (concat "("
-                (with-current-buffer buffer (buffer-string))
-                ")")))
+(defvar mu4e-alert--header-func-save mu4e-alert-mu4e-header-func-var
+  "Saved :header handler from mu4e.")
 
-(defun mu4e-alert--get-mail-sentinel (callback)
-  "Create sentinel for process to get mails from mu, CALLBACK is called with the unread mails."
-  (lambda (process _)
-    ;; If the process has completed successfully parse the mails
-    ;; and execute the callback
-    (when (equal (process-status process) 'exit)
-      (cond ((zerop (process-exit-status process))
-             (funcall callback (mu4e-alert--parse-mails (process-buffer process))))
-            ((= (process-exit-status process) 4)
-             (funcall callback nil))))
+(defvar mu4e-alert--found-func-save mu4e-found-func
+  "Saved :found handler from mu4e.")
 
-    ;; Kill the buffer if the process has terminated
-    (when (memql (process-status process) '(exit signal))
-      (kill-buffer (process-buffer process)))))
+(defvar mu4e-alert--erase-func-save mu4e-erase-func
+  "Saved :erase handler from mu4e.")
 
-(defun mu4e-alert--get-mail-output-buffer ()
-  "Get buffer for storing mails received from mu."
-  (with-current-buffer (get-buffer-create " *mu4e-alert-mails*")
-    (rename-uniquely)
-    (current-buffer)))
+(defvar mu4e-alert--messages
+  "List of messages received by :header callback.")
 
-(defun mu4e-alert--get-mu-unread-emails-1 (callback)
-  "Invoke a mu to get interesting emails and call CALLBACK with the results.
+(defvar mu4e-alert--finding-p nil
+  "Whether `find' request is in progress.")
 
-This is used internally by `mu4e-alert--get-mu-unread-emails' which throttles
-the requests for unread emails."
-  (set-process-sentinel (apply #'start-process
-                               "mu4e-alert-unread-mails"
-                               (mu4e-alert--get-mail-output-buffer)
-                               mu4e-mu-binary
-                               (append (list "find"
-                                             "--nocolor"
-                                             "-o"
-                                             "sexp"
-                                             "--sortfield=d"
-                                             (format "--maxnum=%d" mu4e-alert-max-messages-to-process))
-                                       (when mu4e-headers-skip-duplicates
-                                         (list "-u"))
-                                       (when mu4e-mu-home
-                                         (list (concat "--muhome=" mu4e-mu-home)))
-                                       (split-string mu4e-alert-interesting-mail-query)))
-                        (mu4e-alert--get-mail-sentinel callback)))
+(defun mu4e-alert--erase-func ()
+  "Erase handler for mu process.")
+
+(defun mu4e-alert--found-func (_found)
+  "Found handler for mu process."
+  (mu4e-alert--find-end)
+  (mu4e-alert--email-processor mu4e-alert--messages)
+  (setq mu4e-alert-mu4e-header-func-var mu4e-alert--header-func-save
+        mu4e-found-func mu4e-alert--found-func-save
+        mu4e-erase-func mu4e-alert--erase-func-save))
+
+(defun mu4e-alert--header-func (msg)
+  "Message header handler for mu process.
+MSG argument is message plist."
+  (if mu4e-alert--mu-is-17
+      (setq mu4e-alert--messages msg)
+    (push msg mu4e-alert--messages)))
 
 (defvar mu4e-alert--fetch-timer nil
   "The scheduled fetching of mails from mu.")
 
+(defun mu4e-alert--get-mu-unread-emails-1 ()
+  "Get messages from mu and invoke scheduled callbacks."
+  (when (mu4e-alert--mu4e-proc-running-p-func)
+    (if mu4e-alert--finding-p
+        (setq mu4e-alert--fetch-timer
+              (run-at-time 1.0
+                           nil
+                           #'mu4e-alert--get-mu-unread-emails-1))
+      (setq mu4e-alert-mu4e-header-func-var #'mu4e-alert--header-func
+            mu4e-found-func #'mu4e-alert--found-func
+            mu4e-erase-func #'mu4e-alert--erase-func)
+      (setq mu4e-alert--messages nil)
+      (mu4e-alert--mu4e-proc-find-func mu4e-alert-interesting-mail-query
+                      nil
+                      :date
+                      'descending
+                      mu4e-alert-max-messages-to-process
+                      t
+                      nil))))
+
 (defvar mu4e-alert--callback-queue nil
   "Callbacks queued for running after fetching mails from mu.")
 
@@ -269,18 +326,17 @@ CALLBACK is called with one argument the interesting emails."
   (push callback mu4e-alert--callback-queue)
   (setq mu4e-alert--fetch-timer
         (run-at-time 0.5 nil
-                     #'mu4e-alert--get-mu-unread-emails-1
-                     #'mu4e-alert--email-processor)))
+                     #'mu4e-alert--get-mu-unread-emails-1)))
+
 
-
 
 ;; Mode-line indicator for unread emails
 
-(defvar mu4e-alert-mode-line nil "The mode-line indicator to display the count of unread emails.")
+(defvar mu4e-alert-mode-line "" "The mode-line indicator to display the count of unread emails.")
 
 (defun mu4e-alert-default-mode-line-formatter (mail-count)
   "Default formatter used to get the string to be displayed in the mode-line.
-MAIL-COUNT is the count of mails for which the string is to displayed"
+MAIL-COUNT is the count of mails for which the string is to displayed."
   (when (not (zerop mail-count))
     (concat " "
             (propertize
@@ -305,9 +361,9 @@ MAIL-COUNT is the count of mails for which the string is to displayed"
 (defun mu4e-alert-view-unread-mails ()
   "View unread mails.
 This is primarily used to enable viewing unread emails by default mode-line
-formatter when user clicks on mode-line indicator"
+formatter when user clicks on mode-line indicator."
   (interactive)
-  (mu4e-headers-search mu4e-alert-interesting-mail-query))
+  (mu4e-alert--mu4e-search mu4e-alert-interesting-mail-query))
 
 (defun mu4e-alert-update-mail-count-modeline ()
   "Send a desktop notification about currently unread email."
@@ -316,10 +372,15 @@ formatter when user clicks on mode-line indicator"
                                                                          (length mails)))
                                      (force-mode-line-update))))
 
-
 
 ;; Desktop notifications for unread emails
 
+;;;; Silence byte-compiler warnings for X11-only functions on non-X builds
+(eval-when-compile
+  (declare-function x-window-property "x-win")
+  (declare-function x-change-window-property "x-win")
+  (declare-function x-delete-window-property "x-win"))
+
 ;;;; Setting urgency hint for Emacs frames
 (defun mu4e-alert--set-x-urgency-hint (frame arg)
   "Set window urgency hint for given FRAME.
@@ -364,7 +425,7 @@ This only removes the hints added by `mu4e-alert'"
                            (when (and buffer
                                       (get-buffer-window buffer t))
                              (window-frame (get-buffer-window buffer t))))
-                         (list mu4e~headers-buffer mu4e~view-buffer mu4e~main-buffer-name)))))
+                         (list mu4e-main-buffer-name)))))
 
 (defun mu4e-alert--setup-clear-urgency ()
   "Setup hooks to clear the urgency hooks."
@@ -391,21 +452,28 @@ This only removes the hints added by `mu4e-alert'"
   (cl-remove-if (lambda (mail)
                   (prog1 (and (not mu4e-alert-notify-repeated-mails)
                               (ht-get mu4e-alert-repeated-mails
-                                      (plist-get mail :docid)))
+                                      (plist-get mail :message-id)))
                     (ht-set! mu4e-alert-repeated-mails
-                             (plist-get mail :docid)
+                             (plist-get mail :message-id)
                              t)))
                 mails))
 
+(defun mu4e-alert--message-address (mail key)
+  "Get a string representing address specified by KEY in MAIL."
+  (let ((addr (car (plist-get mail key))))
+    (or
+     (plist-get addr :name)
+     (plist-get addr :email)
+     (car addr)
+     (cdr addr))))
+
 (defun mu4e-alert--get-group (mail)
   "Get the group the given MAIL should be put in.
 
 This is an internal function used by `mu4e-alert-default-mails-grouper'."
   (pcase mu4e-alert-group-by
-    (`:from (or (caar (plist-get mail :from))
-                (cdar (plist-get mail :from))))
-    (`:to (or (caar (plist-get mail :to))
-              (cdar (plist-get mail :to))))
+    (`:from (mu4e-alert--message-address mail :from))
+    (`:to (mu4e-alert--message-address mail :to))
     (`:maildir (plist-get mail :maildir))
     (`:priority (symbol-value (plist-get mail :maildir)))
     (`:flags (s-join ", " (mapcar #'symbol-value
@@ -413,7 +481,7 @@ This is an internal function used by `mu4e-alert-default-mails-grouper'."
 
 (defun mu4e-alert-default-email-count-notification-formatter (mail-count)
   "Default formatter for unread email count.
-MAIL-COUNT is the count of mails for which the string is to displayed"
+MAIL-COUNT is the count of mails for which the string is to displayed."
   (when (not (zerop mail-count))
     (if (= mail-count 1)
         "You have an unread email"
@@ -478,7 +546,8 @@ ALL-MAILS are the all the unread emails"
     (dolist (notification (cl-subseq notifications 0 (min 5 (length notifications))))
       (alert (plist-get notification :body)
              :title (plist-get notification :title)
-             :category "mu4e-alert"))
+             :category "mu4e-alert"
+             :icon mu4e-alert-icon))
     (when notifications
       (mu4e-alert-set-window-urgency-maybe))))
 
@@ -488,7 +557,8 @@ ALL-MAILS are the all the unread emails"
     (alert (funcall mu4e-alert-email-count-notification-formatter
                     mail-count)
            :title mu4e-alert-email-count-title
-           :category "mu4e-alert")))
+           :category "mu4e-alert"
+           :icon mu4e-alert-icon)))
 
 (defun mu4e-alert-notify-unread-mail-async ()
   "Send a desktop notification about currently unread email."
@@ -499,38 +569,46 @@ ALL-MAILS are the all the unread emails"
                                        (when (memql 'subjects mu4e-alert-email-notification-types)
                                          (mu4e-alert-notify-unread-messages new-mails))))))
 
-
+
 
 ;; Tying all the above together
 
-(defadvice mu4e-context-switch (around mu4e-alert-update-mail-count-modeline disable)
-  "Advice `mu4e-context-switch' to update mode-line after changing the context."
-  (let ((context mu4e~context-current))
-    ad-do-it
-    (unless (equal context mu4e~context-current)
+(defun mu4e-alert--context-switch (orig &rest args)
+  "Advice to update mode-line after changing the context.
+ORIG is the original function to be called with ARGS."
+  (let ((context mu4e-alert--mu4e-context-current))
+    (apply orig args)
+    (unless (equal context mu4e-alert--mu4e-context-current)
       (mu4e-alert-update-mail-count-modeline))))
 
+(defun mu4e-alert--find-start (&rest _)
+  "Advice to track `find' request start."
+  (setq mu4e-alert--finding-p t))
+
+(defun mu4e-alert--find-end (&rest _)
+  "Advice to track `find' request start."
+  (setq mu4e-alert--finding-p nil))
+
+(advice-add (symbol-function #'mu4e-alert--mu4e-proc-find-func) :before #'mu4e-alert--find-start)
+(advice-add mu4e-alert--found-func-save :before #'mu4e-alert--find-end)
+
 ;;;###autoload
 (defun mu4e-alert-enable-mode-line-display ()
   "Enable display of unread emails in mode-line."
   (interactive)
   (add-to-list 'global-mode-string '(:eval mu4e-alert-mode-line) t)
-  (add-hook 'mu4e-view-mode-hook #'mu4e-alert-update-mail-count-modeline)
   (add-hook 'mu4e-index-updated-hook #'mu4e-alert-update-mail-count-modeline)
-  (add-hook 'mu4e-msg-changed-hook #'mu4e-alert-update-mail-count-modeline)
-  (ad-enable-advice #'mu4e-context-switch 'around 'mu4e-alert-update-mail-count-modeline)
-  (ad-activate #'mu4e-context-switch)
+  (add-hook 'mu4e-message-changed-hook #'mu4e-alert-update-mail-count-modeline)
+  (advice-add #'mu4e-context-switch :around #'mu4e-alert--context-switch)
   (mu4e-alert-update-mail-count-modeline))
 
 (defun mu4e-alert-disable-mode-line-display ()
   "Disable display of unread emails in mode-line."
   (interactive)
   (setq global-mode-string (delete '(:eval mu4e-alert-mode-line) global-mode-string))
-  (remove-hook 'mu4e-view-mode-hook #'mu4e-alert-update-mail-count-modeline)
   (remove-hook 'mu4e-index-updated-hook #'mu4e-alert-update-mail-count-modeline)
-  (remove-hook 'mu4e-msg-changed-hook #'mu4e-alert-update-mail-count-modeline)
-  (ad-disable-advice #'mu4e-context-switch 'around 'mu4e-alert-update-mail-count-modeline)
-  (ad-deactivate #'mu4e-context-switch))
+  (remove-hook 'mu4e-message-changed-hook #'mu4e-alert-update-mail-count-modeline)
+  (advice-remove #'mu4e-context-switch #'mu4e-alert--context-switch))
 
 ;;;###autoload
 (defun mu4e-alert-enable-notifications ()
